#!/usr/bin/python3

import sys
import os
import subprocess
import re
import shutil
from glob import glob
import xml.etree.ElementTree as ET
from exceptions import exceptions
from PIL import Image
from games import games

ask = input("Start processing? ")
if ask.lower() in ["yes", "y"]:
    pass
else:
    sys.exit(0)


catalog_location = sys.argv[1]

lib_files = glob("lib*.deb")
deb_files = glob("*.deb")

xml = open(catalog_location, "w")
lib_xml = open("libs.xml", "w")
games_xml = open("games.xml", "w")
xml.write("<catalog>")
lib_xml.write("<catalog>")
games_xml.write("<catalog>")

directory = os.getcwd()
os.chdir(directory)
if os.path.isdir("icons"):
    shutil.rmtree("icons")
os.mkdir("icons")
shutil.copyfile("default.png", "icons/default.png")
def cleaner(string):
    string = string.replace("&", "and")
    string = string.replace("<", "").replace(">", "")
    string = string.replace("\"", "'")
    return string

def get_file_size(file_path):
    size = subprocess.check_output("du -sh \"{}\"".format(file_path), shell=True).split()[0].decode('utf-8')
    
    size = size.replace(file_path, "").rstrip().replace(".0", "")
    
    if size.endswith("K"):
        size = size.replace("K", " KB")
    elif size.endswith("M"):
        size = size.replace("M", " MB")
    elif size.endswith("G"):
        size = size.replace("G", " GB")
    return size

for deb_file in deb_files:
    
    if deb_file in lib_files:
        continue

    try:

        deb_name = deb_file.replace(".deb", "")

        control_dir = os.path.join(deb_name, "DEBIAN")
        data_dir = os.path.join(deb_name, "data")

        os.makedirs(deb_name, exist_ok=True)

        subprocess.call(f"dpkg-deb -e \"{deb_file}\" \"{control_dir}\"", shell=True)
        subprocess.call(f"dpkg-deb -x \"{deb_file}\" \"{data_dir}\"", shell=True)

        path_80 = os.path.join(data_dir, "usr", "share", "icons", "hicolor", "80x80", "apps")
        path_64 = os.path.join(data_dir, "usr", "share", "icons", "hicolor", "64x64", "apps")

        with open(os.path.join(control_dir, "control"), "r") as f:
            for line in f.readlines():
                match = re.search(r'^Version:\s*(.+)', line)
                if match:
                    version = cleaner(match.group(1))

                match = re.search(r'^Package:\s*(.+)', line)
                if match:
                    package = cleaner(match.group(1))
            
                match = re.search(r'^Maemo-Display-Name:\s*(.+)', line)
                if match:
                    display_name = cleaner(match.group(1))

                if package in exceptions.keys():
                    display_name = exceptions[package]

                match = re.search(r'^Maintainer:\s*(.+)', line)
                if match:
                    dev = match.group(1)
                    email = re.search(r'\s*<.*>', dev)
                    if email:
                        email = email.group(0)
                        dev = dev.replace(email, "")
                    dev = cleaner(dev)
                size = get_file_size(deb_file)

        try:
            if not display_name:
                display_name = package
        except NameError:
            display_name = package

        if os.path.isdir(path_64):
            icon_dir = "64"
        elif os.path.isdir(path_80):
            icon_dir = "80"
        else:
            icon_dir = None

        if icon_dir == "64":
            icons = os.listdir(path_64)
            if icons:
                icon_path = os.path.join(path_64, icons[0])
            else:
                icon_path = None

            shutil.copy(icon_path, "icons/{}_temp.png".format(package))

        elif icon_dir == "80":
            icons = os.listdir(path_80)
            if icons:
                icon_path = os.path.join(path_80, icons[0])
            else:
                icon_path = None

            shutil.copy(icon_path, "icons/{}_temp.png".format(package))

        else:
            icon_path = None

        if icon_path:
            icon_opened = Image.open("icons/{}_temp.png".format(package))
            new_size = (64, 64)
            small_icon = icon_opened.resize(new_size)
            small_icon.save("icons/{}.png".format(package))
            os.remove("icons/{}_temp.png".format(package))

        shutil.rmtree(deb_name)

        if icon_path:
            icon = "{}.png".format(package)
        else:
            icon = "default.png"

        if package in games:
            games_xml.write("""
    <app>
        <data deb="{}" name="{}" ver="{}" package="{}" dev="{}" size="{}" icon="{}" />
    </app>""".format(deb_file, display_name, version, package, dev, size, icon))

        else:
            xml.write("""
    <app>
        <data deb="{}" name="{}" ver="{}" package="{}" dev="{}" size="{}" icon="{}" />
    </app>""".format(deb_file, display_name, version, package, dev, size, icon))

        #txt.write(display_name + "\n")

        deb_file = None
        display_name = None
        version = None
        package = None
        dev = None
        size = None
        
    except Exception as e:
        exc_type, exc_obj, exc_tb = sys.exc_info()
        fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
        print(exc_type, fname, exc_tb.tb_lineno, deb_file)
        continue

for lib_file in lib_files:

    try:

        lib_name = lib_file.replace(".deb", "")

        control_dir = os.path.join(lib_name, "DEBIAN")

        os.makedirs(lib_name, exist_ok=True)

        subprocess.call(f"dpkg-deb -e \"{lib_file}\" \"{control_dir}\"", shell=True)

        with open(os.path.join(control_dir, "control"), "r") as f:
            for line in f.readlines():
                match = re.search(r'^Version:\s*(.+)', line)
                if match:
                    version = cleaner(match.group(1))

                match = re.search(r'^Package:\s*(.+)', line)
                if match:
                    package = cleaner(match.group(1))

                match = re.search(r'^Maemo-Display-Name:\s*(.+)', line)
                if match:
                    display_name = cleaner(match.group(1))

                if package in exceptions.keys():
                    display_name = exceptions[package]

                match = re.search(r'^Maintainer:\s*(.+)', line)
                if match:
                    dev = match.group(1)
                    email = re.search(r'\s*<.*>', dev)
                    if email:
                        email = email.group(0)
                        dev = dev.replace(email, "")
                    dev = cleaner(dev)
                size = get_file_size(lib_file)

        try:
            if not display_name:
                display_name = package
        except NameError:
            display_name = package

        shutil.rmtree(lib_name)

        lib_xml.write("""
    <app>
        <data deb="{}" name="{}" ver="{}" package="{}" dev="{}" size="{}" />
    </app>""".format(lib_file, display_name, version, package, dev, size))

        lib_file = None
        display_name = None
        version = None
        package = None
        dev = None
        size = None

    except Exception as e:
        exc_type, exc_obj, exc_tb = sys.exc_info()
        fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
        print(exc_type, fname, exc_tb.tb_lineno, deb_file)
        continue

xml.write("\n</catalog>")
lib_xml.write("\n</catalog>")
games_xml.write("\n</catalog>")
xml.close()
lib_xml.close()
games_xml.close()
